/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Eric Reiss, Michael J. Schnieders
 * @version 4.2
 */

package ffe.behaviors;

import java.util.Enumeration;
import javax.media.j3d.*;

import java.awt.event.MouseEvent;

/**
 * The MouseBehavior class is the Base class for all mouse manipulators.
 */
public abstract class MouseBehavior
   extends Behavior{
   protected WakeupCriterion[] mouseEvents;
   protected WakeupOr mouseCriterion;
   protected Behavior poster;
   protected int id;
   protected WakeupOnBehaviorPost postCriterion;
   protected int x, y;
   protected int x_last, y_last;
   protected TransformGroup transformGroup;
   protected Transform3D transformX;
   protected Transform3D transformY;
   protected Transform3D currXform;
   protected boolean buttonPress = false;
   protected boolean reset = false;
   protected boolean invert = false;
   protected boolean wakeUp = false;
   protected int flags = 0;
   protected TransformGroup ViewerTG;

   public MouseBehavior(int format, TransformGroup VPTG){
      super();
      flags = format;
      ViewerTG = VPTG;
      currXform = new Transform3D();
      transformX = new Transform3D();
      transformY = new Transform3D();
      reset = true;
   }

   public MouseBehavior(int format, TransformGroup VPTG, Behavior b, int i){
      this(format, VPTG);
      poster = b;
      id = i;
   }

   /*
    * Initializes the behavior.
    */
   public void initialize(){
      mouseEvents = new WakeupCriterion[3];
      mouseEvents[0] = new WakeupOnAWTEvent(MouseEvent.MOUSE_DRAGGED);
      mouseEvents[1] = new WakeupOnAWTEvent(MouseEvent.MOUSE_PRESSED);
      mouseEvents[2] = new WakeupOnAWTEvent(MouseEvent.MOUSE_RELEASED);
      mouseCriterion = new WakeupOr(mouseEvents);
      if (poster == null){
         wakeupOn(mouseCriterion);
      } else{
         postCriterion = new WakeupOnBehaviorPost(poster, id);
         wakeupOn(postCriterion);
      }
      x = 0;
      y = 0;
      x_last = 0;
      y_last = 0;
   }

   /*
    * Handles mouse events
    */
   public void processMouseEvent(MouseEvent evt){
      if (evt.getID() == MouseEvent.MOUSE_PRESSED){
         buttonPress = true;
      } else if (evt.getID() == MouseEvent.MOUSE_RELEASED){
         buttonPress = false;
         wakeUp = false;
      }
   }

   public abstract void processStimulus(Enumeration criteria);

   /*
    * Swap a new transformGroup replacing the old one. This allows
    * manipulators to operate on different nodes.
    *
    * @param transformGroup The *new* transform group to be manipulated.
    */
   Transform3D t3d = new Transform3D();
   public void setTransformGroup(TransformGroup t){
      transformGroup = t;
      currXform = new Transform3D();
      transformX = new Transform3D();
      transformY = new Transform3D();
      reset = true;
   }

   /*
    * Manually wake up the behavior. If MANUAL_WAKEUP flag was set upon
    * creation, you must wake up this behavior each time it is handled.
    */
   public void wakeup(){
      wakeUp = true;
   }

   /*
    * Set this flag if you want to manually wakeup the behavior.
    */
   public static final int MANUAL_WAKEUP = 0x1;

   /*
    * Set this flag if you want to invert the inputs.  This is useful when
    * the transform for the view platform is being changed instead of the
    * transform for the object.
    */
   public static final int INVERT_INPUT = 0x2;

}

/*
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */
