/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Eric Reiss, Michael J. Schnieders
 * @version 4.2
 */

package ffe.behaviors;

import java.util.Enumeration;
import javax.media.j3d.*;
import javax.vecmath.Vector3d;

import java.awt.AWTEvent;
import java.awt.event.MouseEvent;

/**
 * The MouseZoom class implements a Mouse Zoom behavior.
 */
public class MouseZoom
   extends MouseBehavior{
   double z_factor = 0.0002;
   Vector3d translation = new Vector3d();
   private MouseBehaviorCallback callback = null;

   public MouseZoom(TransformGroup baseTransform, int flags, Bounds bounds, TransformGroup VPTG){
      super(flags, VPTG);
      setTransformGroup(baseTransform);
      setSchedulingBounds(bounds);
   }

   public void initialize(){
      super.initialize();
      if ((flags & INVERT_INPUT) == INVERT_INPUT){
         z_factor *= -1;
         invert = true;
      }
   }

   /*
    * Return the y-axis movement multipler.
    */
   public double getFactor(){
      return z_factor;
   }

   /*
    * Set the y-axis movement multipler with factor.
    */
   public void setFactor(double factor){
      z_factor = factor;
   }

   public void processStimulus(Enumeration criteria){
      WakeupCriterion wakeup;
      AWTEvent[] event;
      int id;
      int dx, dy;
      while (criteria.hasMoreElements()){
         wakeup = (WakeupCriterion) criteria.nextElement();
         if (wakeup instanceof WakeupOnAWTEvent){
            event = ((WakeupOnAWTEvent) wakeup).getAWTEvent();
            for (int i = 0; i < event.length; i++){
               processMouseEvent((MouseEvent) event[i]);
               id = event[i].getID();
               MouseEvent mevent = (MouseEvent) event[i];
               if ((id == MouseEvent.MOUSE_DRAGGED) &&
                  ((mevent.getModifiersEx() &
                  MouseEvent.BUTTON2_DOWN_MASK) == MouseEvent.BUTTON2_DOWN_MASK)){
                  x = ((MouseEvent) event[i]).getX();
                  y = ((MouseEvent) event[i]).getY();
                  dx = x - x_last;
                  dy = y - y_last;
                  if (!reset){
                     transformGroup.getTransform(currXform);
                     double z = ( -1.0) * dy * z_factor;
                     double scale = currXform.getScale() + z;
                     if (scale > 0){
                        currXform.setScale(scale);
                        transformGroup.setTransform(currXform);
                        transformChanged(currXform);
                     }
                     if (callback != null){
                        callback.transformChanged(MouseBehaviorCallback.ZOOM, currXform);
                     }
                  } else{
                     reset = false;
                  }
                  x_last = x;
                  y_last = y;
               } else if (id == MouseEvent.MOUSE_PRESSED){
                  x_last = ((MouseEvent) event[i]).getX();
                  y_last = ((MouseEvent) event[i]).getY();
               }
            }
         }
      }
      wakeupOn(mouseCriterion);
   }

   public void transformChanged(Transform3D transform){}

   /*
    * The transformChanged method in the callback class will
    * be called every time the transform is updated
    */
   public void setupCallback(MouseBehaviorCallback c){
      callback = c;
   }
}

/*
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */
