/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Eric Reiss, Michael J. Schnieders
 * @version 4.2
 */

package ffe.behaviors;

import javax.media.j3d.*;

import java.awt.event.MouseEvent;

import com.sun.j3d.utils.picking.PickResult;

import ffe.panels.*;

/**
 * The PickTranslateBehavior class implements a translation behavior on a
 * picked scenegraph object.
 */
public class PickTranslateBehavior
   extends PickMouseBehavior
   implements MouseBehaviorCallback{

   public MouseTranslate translate;
   private PickingCallback callback = null;
   private TransformGroup currentTG;
   private MainPanel fpanel = null;

   public PickTranslateBehavior(BranchGroup root, Canvas3D canvas, Bounds bounds,
      TransformGroup VPTG, int pickMode){
      super(canvas, root, bounds);
      translate = new MouseTranslate(MouseBehavior.MANUAL_WAKEUP, VPTG);
      translate.setTransformGroup(currGrp);
      translate.setFactor(0.1);
      currGrp.addChild(translate);
      translate.setSchedulingBounds(bounds);
      setSchedulingBounds(bounds);
      pickCanvas.setMode(pickMode);
   }

   public PickTranslateBehavior(BranchGroup bg, Canvas3D canvas, Bounds bounds, TransformGroup VPTG,
      int pickMode, MainPanel f){
      this(bg, canvas, bounds, VPTG, pickMode);
      fpanel = f;
   }

   /*
    * Return the pickMode component of this PickTranslateBehavior.
    */
   public int getPickMode(){
      return pickCanvas.getMode();
   }

   /*
    * Sets the pickMode component of this PickTranslateBehavior to the value of
    * the passed pickMode.
    * @param pickMode the pickMode to be copied.
    */
   public void setPickMode(int pickMode){
      pickCanvas.setMode(pickMode);
   }

   /*
    * Register the class @param callback to be called each
    * time the picked object moves
    */
   public void setupCallback(PickingCallback callback){
      this.callback = callback;
      if (callback == null){
         translate.setupCallback(null);
      } else{
         translate.setupCallback(this);
      }
   }

   public void transformChanged(int type, Transform3D transform){
      callback.transformChanged(PickingCallback.TRANSLATE, currentTG);
   }

   public void transformClicked(int type, Transform3D transform){
      callback.transformClicked(PickingCallback.TRANSLATE, currentTG);
   }

   public void transformDoubleClicked(int type, Transform3D transform){
      callback.transformDoubleClicked(PickingCallback.TRANSLATE, currentTG);
   }

   /*
    * Update the scene to manipulate any nodes. This is not meant to be
    * called by users. Behavior automatically calls this. You can call
    * this only if you know what you are doing.
    *
    * @param xpos Current mouse X pos.
    * @param ypos Current mouse Y pos.
    */
   public void updateScene(int xpos, int ypos){
      TransformGroup tg = null;
      if ((mevent.getModifiersEx() & MouseEvent.BUTTON3_DOWN_MASK) == MouseEvent.BUTTON3_DOWN_MASK){
         pickCanvas.setShapeLocation(xpos, ypos);
         PickResult r = pickCanvas.pickClosest();
         if (r != null){
            Graphics3D g = fpanel.getGraphics3D();
            if (g == null || g.getPickMode() == Graphics3D.SCREEN){
               tg = (TransformGroup) r.getNode(PickResult.TRANSFORM_GROUP);
            } else{
               if (fpanel.getRoot() == null || fpanel.getTree().getActive() == null){
                  return;
               }
               tg = fpanel.getTree().getActive().getTransformGroup();
            }
            if ((tg != null) && (tg.getCapability(TransformGroup.ALLOW_TRANSFORM_READ)) &&
               (tg.getCapability(TransformGroup.ALLOW_TRANSFORM_WRITE))){
               translate.setTransformGroup(tg);
               translate.wakeup();
               currentTG = tg;
               if (callback != null){
                  callback.transformClicked(PickingCallback.TRANSLATE, currentTG);
               }
            } else if (callback != null){
               callback.transformChanged(PickingCallback.NO_PICK, null);
            }
         }
      }
   }
}

/*
 * Copyright (c) 1996-1998 Sun MicroFSystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */
