/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.lang;

import java.util.ListIterator;

/**
 * The Joint class contains the geometry produced by the FGroup Joiner method.
 */
public final class Joint
   extends FNode{
   /**
    * Bond objects are attached to this node
    */
   public FNode bonds;

   /**
    * Angle objects are attached to this node
    */
   public FNode angles;

   /**
    * Dihedral objects are attached to this node
    */
   public FNode dihedrals;

   /**
    * Improper objects are attached to this node
    */
   public FNode impropers;

   /**
    * One of two Moieties forming this Joint
    */
   public FGroup m1, m2;

   /**
    * Default Constructor
    */
   public Joint(){
      super("Joint");
      m1 = m2 = null;
   }

   /**
    * Constructs a Joint between Moieties m1 and m2, where the Bond, Angle and Dihedral geometry
    * objects have been attached to FNode b,a and d, respectively
    */
   public Joint(FGroup m1, FGroup m2, FNode b, FNode a, FNode d){
      super("Joint");
      this.m1 = m1;
      this.m2 = m2;
      bonds = b;
      angles = a;
      dihedrals = d;
      add(bonds);
      add(angles);
      add(dihedrals);
   }

   public void merge(Joint j){
      if (m1 != j.m1 || m1 != j.m2){
         return;
      }
      if (bonds == null){
         bonds = new FNode();
         add(bonds);
      }
      if (angles == null){
         angles = new FNode();
         add(angles);
      }
      if (dihedrals == null){
         dihedrals = new FNode();
         add(dihedrals);
      }

      for (ListIterator li = j.getTermList().listIterator(); li.hasNext(); ){
         Term t = (Term) li.next();
         t.removeFromParent();
         if (t instanceof Bond){
            bonds.add(t);
         } else if (t instanceof Angle){
            angles.add(t);
         } else if (t instanceof Dihedral){
            dihedrals.add(t);
         }
      }
   }

   /**
    * Overidden toString method returns: "Joint: m1 Name - m2 Name"
    */
   public String toString(){
      String s = new String(m1.toString() + "-" + m2.toString());
      return s;
   }
}
