/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.lang;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * The MMAtom class represents one molecular mechanics atom type.
 */
public final class MMAtom
   implements Serializable{
   /**
    * Key
    */
   public String type;

   /**
    * Class
    */
   public boolean stused;
   public String specifictype;

   /**
    * Basic Type (ie CH3/CH2 etc)
    */
   public String atoms;

   /**
    * Description of the atom's bonding environment
    */
   public String description;

   /**
    * Atomic Number
    */
   public int atomic_num;

   /**
    * Atomic Mass
    */
   public float mass;

   /**
    * Atomic Hybridization
    */
   public int hybridization;

   /**
    * Sigma Value for van der Waals interactions
    */
   public float sigma;

   /**
    * Epsilon Value for van der Waals interactions
    */
   public float epsilon;

   /**
    * Atomic partial charge
    */
   public float charge;
   public boolean used = false;

   public static ArrayList mmatompool = new ArrayList();

   /**
    * Molecular Mechanics Atom Constructor, where k is the key; type are atoms represented; name the
    * identity; a the atomic_num; m the mass; h the hybridization; v1 is sigma; v2 is epsilon;
    * c is charge
    */
   public MMAtom(String k, String t, String name, int a, float m, int h, float v1, float v2,
      float c){
      type = k;
      atoms = t;
      description = name;
      atomic_num = a;
      mass = m;
      hybridization = h;
      sigma = v1;
      epsilon = v2;
      charge = c;
      specifictype = null;
      stused = false;
      used = true;
   }

   public void reInit(String k, String t, String name, int a, float m, int h, float v1, float v2,
      float c){
      type = k;
      atoms = t;
      description = name;
      atomic_num = a;
      mass = m;
      hybridization = h;
      sigma = v1;
      epsilon = v2;
      charge = c;
      specifictype = null;
      stused = false;
      used = true;
   }

   public MMAtom(String k, String c, String t, String name, int a, float m, int h){
      type = k;
      specifictype = c;
      stused = true;
      atoms = t;
      description = name;
      atomic_num = a;
      mass = m;
      hybridization = h;
      used = true;
   }

   public void reInit(String k, String c, String t, String name, int a, float m, int h){
      type = k;
      specifictype = c;
      stused = true;
      atoms = t;
      description = name;
      atomic_num = a;
      mass = m;
      hybridization = h;
      used = true;
   }

   public MMAtom(String k, String t, String name, int a, float m, int h){
      type = k;
      atoms = t;
      description = name;
      atomic_num = a;
      mass = m;
      hybridization = h;
      specifictype = null;
      stused = false;
      used = true;
   }

   public void reInit(String k, String t, String name, int a, float m, int h){
      type = k;
      atoms = t;
      description = name;
      atomic_num = a;
      mass = m;
      hybridization = h;
      specifictype = null;
      stused = false;
      used = true;
   }

   public void destroy(){
      type = null;
      atoms = null;
      description = null;
      atomic_num = 0;
      mass = 0;
      hybridization = 0;
      sigma = 0;
      epsilon = 0;
      charge = 0;
      specifictype = null;
      stused = false;
      used = false;
      mmatompool.add(this);
   }

   public String toString(){
      String s;
      if (specifictype != null){
         s = new String("MMAtom: " + type + " " + specifictype + " " + description + " " +
             atomic_num);
      } else{
         s = new String("MMAtom: " + type + " " + description + " " + atomic_num);
      }
      return s;
   }

   public void print(){
      System.out.println("" + toString());
   }
}
