/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.lang;

/**
 * The Molecule class is a general container used for simple compounds or in cases where more specialized classes have not been implemented.
 */
public class Molecule
   extends FGroup{

   public Molecule(){
   }

   public Molecule(String name, boolean bk){
      super(name, bk);
   }

   /**
    * Allows adding Atom FNodes to the Molecule.
    */
   public void addFNode(FNode o){
      if (o instanceof Atom){
         getFGroups().add(o);
         setFinalized(false);
      } else{
         System.out.println("Can not add " + o.getClass() + " to a Molecule, not of type Atom");
      }
   }

   public void finalize(boolean finalizeGeometry){
      setFinalized(false);
      getFGroups().setName("Atoms: " + getAtomList().size());
      if (finalizeGeometry){
         if (!getBondsKnown()){
            ConstructGeometry();
         } else{
            collectBonds();
         }
         removeLeaves();
      }
      //findDangelingAtoms();
      setCenter(findCenter(false));
      setFinalized(true);
   }

}
