/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.lang;

import java.util.Enumeration;
import javax.media.j3d.BranchGroup;

import ffe.panels.MainPanel;

/**
 * The Root class is the base of the Force Field Explorer data structure.
 */
public final class Root
   extends FNode{

   MainPanel fpanel;

   /**
    * Default Root Constructor
    */public Root(MainPanel f){
      super("Systems");
      fpanel = f;
   }

   public void addFSystem(FSystem s){
      add(s);
   }

   public long getAtomCount(){
      Enumeration e = children();
      long count = 0;
      while (e.hasMoreElements()){
         FSystem t = (FSystem) e.nextElement();
         count += t.getAtomList().size();
      }
      return count;
   }

   public long getBondCount(){
      Enumeration e = children();
      long count = 0;
      while (e.hasMoreElements()){
         FSystem t = (FSystem) e.nextElement();
         count += t.getBondList().size();
      }
      return count;
   }

   public void attachall(){
      for (Enumeration e = children(); e.hasMoreElements(); ){
         FSystem s = (FSystem) e.nextElement();
         if (s.isVisible()){
            BranchGroup bg = s.getBranchGroup();
            if (!bg.isLive()){
               fpanel.graphics.attach(bg);
            }
         }
      }
   }

   public double getExtent(){
      FSystem s;
      double extent, max = 1;
      for (int i = 0; i < getChildCount(); i++){
         s = (FSystem) getChildAt(i);
         extent = s.getExtent();
         if (extent > max){
            max = extent;
         }
      }
      return max;
   }

   public String toString(){
      return "Systems";
   }
}
