/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.panels;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * The FMenuBar class creates the Force Field Explorer Menu Bar */
public class FMenuBar
   extends JMenuBar{

   private MainPanel main;
   private ffe.panels.Graphics3D graphics;

   // Main Menubar
   private JMenu mFile = new JMenu("File");
   private JMenu mSelection = new JMenu("Selection");
   private JMenu mDisplay = new JMenu("Display");
   private JMenu mColor = new JMenu("Color");
   private JMenu mOptions = new JMenu("Options");
   private JMenu mPicking = new JMenu("Picking");
   private JMenu mTrajectory = new JMenu("Trajectory");
   private JMenu mSimulation = new JMenu("Simulation");
   private JMenu mExport = new JMenu("Export");
   private JMenu mWindow = new JMenu("Window");
   private JMenu mHelp = new JMenu("Help");

   // File Menu
   private ClassLoader loader = getClass().getClassLoader();
   private ImageIcon icOpen = new ImageIcon(loader.getResource("ffe/icons/fileopen.png"));
   private ImageIcon icSaveAs = new ImageIcon(loader.getResource("ffe/icons/filesaveas.png"));
   private ImageIcon icClose = new ImageIcon(loader.getResource("ffe/icons/fileclose.png"));
   private ImageIcon icExit = new ImageIcon(loader.getResource("ffe/icons/exit.png"));
   private ImageIcon icBlank = new ImageIcon(loader.getResource("ffe/icons/blank.gif"));

   private JMenuItem miOpen = new JMenuItem("Open...", icOpen);
   private JMenuItem miMerge = new JMenuItem("Merge", icBlank);
   private JMenuItem miSaveXYZ = new JMenuItem("Save as...", icSaveAs);
   private JMenuItem miOpenPDB = new JMenuItem("Download from PDB...", icBlank);
   private JMenuItem miLoadRestart = new JMenuItem("Load Restart Data...", icBlank);
   private JMenuItem miLoadInduced = new JMenuItem("Load Induced Data...", icBlank);
   private JMenuItem miClose = new JMenuItem("Close", icClose);
   private JMenuItem miCloseAll = new JMenuItem("Close All", icBlank);
   private JMenuItem miExit = new JMenuItem("Exit", icExit);

   // Edit Menu
   private ImageIcon icColor = new ImageIcon(loader.getResource("ffe/icons/color.png"));
   private ImageIcon icHighlight = new ImageIcon(loader.getResource("ffe/icons/highlight.png"));
   private JCheckBoxMenuItem miHighlight = new JCheckBoxMenuItem("Highlight Selections",
                                           icHighlight);
   private JMenuItem miRestrict = new JMenuItem("Restrict to Selections", icBlank);
   private JMenuItem miSelectAll = new JMenuItem("Select All", icBlank);
   private JMenuItem miSelectionColor = new JMenuItem("Set Selection Color...", icColor);
   private JCheckBoxMenuItem miLabelResidues = new JCheckBoxMenuItem("Label Selected Residues",
                                               icBlank);
   private JCheckBoxMenuItem miLabelAtoms = new JCheckBoxMenuItem("Label Selected Atoms", icBlank);
   private JMenuItem miLabelSize = new JMenuItem("Set Label Font Size...", icBlank);
   private JMenuItem miLabelFontColor = new JMenuItem("Set Label Font Color...", icColor);

   // Display Menu
   private JMenuItem miWireframe = new JMenuItem("Wireframe", icBlank);
   private JMenuItem miSpacefill = new JMenuItem("Spacefill", icBlank);
   private JMenuItem miInvisible = new JMenuItem("Invisible", icBlank);
   private JMenuItem miBall = new JMenuItem("Ball & Stick", icBlank);
   private JMenuItem miTube = new JMenuItem("Tube", icBlank);
   private JMenuItem miRMIN = new JMenuItem("RMIN", icBlank);

   private JMenuItem miPoints = new JMenuItem("Points", icBlank);
   private JMenuItem miLines = new JMenuItem("Lines", icBlank);
   private JMenuItem miFill = new JMenuItem("Fill", icBlank);

   private ImageIcon icPreferences = new ImageIcon(loader.getResource("ffe/icons/settings.png"));
   private JMenuItem miPreferences = new JMenuItem("Preferences...", icPreferences);

   // Color Menu
   private JMenuItem miMonochrome = new JMenuItem("Monochrome", icBlank);
   private JMenuItem miAtomColor = new JMenuItem("CPK", icBlank);
   private JMenuItem miResidueColor = new JMenuItem("Residue", icBlank);
   private JMenuItem miStructureColor = new JMenuItem("Structure", icBlank);
   private JMenuItem miPolymerColor = new JMenuItem("Polymer", icBlank);
   private JMenuItem miChargeColor = new JMenuItem("Partial Charge", icBlank);
   private JMenuItem miVectorColor = new JMenuItem("Vector Magnitude", icBlank);
   private JMenuItem miUsersColor = new JMenuItem("User Color", icBlank);
   private JMenuItem miApplyUserColor = new JMenuItem("Apply User Color to Selections", icBlank);
   private JMenuItem miUserColor = new JMenuItem("Set User Color...", icColor);

   // Options Menu
   private ImageIcon icHome = new ImageIcon(loader.getResource("ffe/icons/gohome.png"));
   private ImageIcon icTree = new ImageIcon(loader.getResource("ffe/icons/view_tree.png"));
   private ImageIcon icMouse = new ImageIcon(loader.getResource("ffe/icons/mouse.png"));

//private ImageIcon icformat = new ImageIcon(loader.getResource("ffe/icons/images.png"));

   private JMenu mActive = new JMenu("Active System");
   private JMenuItem miRemoveRotation = new JMenuItem("Reset Rotation");
   private JMenuItem miRemoveTranslation = new JMenuItem("Reset Translation");
   private JMenuItem miRemoveRotTrans = new JMenuItem("Reset Rotation and Translation");
   private JMenuItem miRotateOnCOM = new JMenuItem("Rotate About Center");
   private JMenuItem miRotateOnPick = new JMenuItem("Rotate About Pick");
   private JMenu mMode = new JMenu("Drag");
   private JRadioButtonMenuItem miActive = new JRadioButtonMenuItem("Active System", icTree);
   private JRadioButtonMenuItem miCanvas = new JRadioButtonMenuItem("System Below Mouse", icMouse);
   private ButtonGroup bgMode = new ButtonGroup();

   private JMenuItem miZoom = new JMenuItem("Reset Global Zoom", icBlank);
   private JMenuItem miTranslation = new JMenuItem("Reset Global Translation", icBlank);
   private JMenuItem miRotation = new JMenuItem("Reset Global Rotation", icBlank);
   private JMenuItem miView = new JMenuItem("Reset Global View", icHome);

   //private JMenuItem miOrigin = new JMenuItem("Global Rotate About Pick", icblank);
   //private JMenuItem miResetOrigin = new JMenuItem("Global Rotate About Origin", icblank);

   private JMenuItem miHydrogens = new JMenuItem("Toggle Hydrogens", icBlank);

   //private JMenuItem miVRML = new JMenuItem("Toggle VRML", icblank);
   //private JMenuItem miVectors = new JMenuItem("Toggle Vectors", icBlank);
   //private JMenu mVector = new JMenu("Vector Type");
   //private JMenuItem miInduced = new JMenuItem("Induced Dipole");
   //private JMenuItem miForces = new JMenuItem("Force");
   //private JMenuItem miVelocity = new JMenuItem("Velocity");
   //private JMenuItem miAcceleration = new JMenuItem("Acceleration");
   //private JMenu mLength = new JMenu("Vector Length");
   //private JMenuItem miVUnit = new JMenuItem("Unit");
   //private JMenuItem miVRelative = new JMenuItem("Relative");
   //private JMenuItem miVAbsolute = new JMenuItem("Absolute");

   private ImageIcon icFullScreen = new ImageIcon(loader.getResource("ffe/icons/fullscreen.png"));
   private ImageIcon icBackground = new ImageIcon(loader.getResource("ffe/icons/background.png"));
   private JMenuItem miFullScreen = new JMenuItem("Full Screen", icFullScreen);
   private JMenuItem miBackgroundColor = new JMenuItem("Set Background Color...", icBackground);

   // Picking Menu
   private ImageIcon icSelect = new ImageIcon(loader.getResource("ffe/icons/pick.png"));
   private JCheckBoxMenuItem miPicking = new JCheckBoxMenuItem("Graphics Picking", icSelect);
   private JMenuItem miPickColor = new JMenuItem("Set Graphics Picking Color...", icColor);
   private JRadioButtonMenuItem rbAtom = new JRadioButtonMenuItem("Atom", icBlank);
   private JRadioButtonMenuItem rbBond = new JRadioButtonMenuItem("Bond", icBlank);
   private JRadioButtonMenuItem rbAngle = new JRadioButtonMenuItem("Angle", icBlank);
   private JRadioButtonMenuItem rbDihedral = new JRadioButtonMenuItem("Dihedral", icBlank);
   private JRadioButtonMenuItem rbResidue = new JRadioButtonMenuItem("Residue", icBlank);
   private JRadioButtonMenuItem rbPolymer = new JRadioButtonMenuItem("Polymer", icBlank);
   private JRadioButtonMenuItem rbMolecule = new JRadioButtonMenuItem("Molecule", icBlank);
   private JRadioButtonMenuItem rbSystem = new JRadioButtonMenuItem("System", icBlank);
   private JRadioButtonMenuItem rbMeasureDistance = new JRadioButtonMenuItem("Measure Distance",
      icBlank);
   private JRadioButtonMenuItem rbMeasureAngle = new JRadioButtonMenuItem("Measure Angle", icBlank);
   private JRadioButtonMenuItem rbMeasureDihedral = new JRadioButtonMenuItem("Measure Dihedral",
      icBlank);
   private ButtonGroup bgLevel = new ButtonGroup();

   // Trajectory Menu
   private ImageIcon icOscillate = new ImageIcon(loader.getResource("ffe/icons/oscillate.png"));
   private ImageIcon icRelease = new ImageIcon(loader.getResource("ffe/icons/release.png"));
   private ImageIcon icNetwork = new ImageIcon(loader.getResource("ffe/icons/network.png"));
   private ImageIcon icLocal = new ImageIcon(loader.getResource("ffe/icons/local.png"));

   private ImageIcon icForward = new ImageIcon(loader.getResource("ffe/icons/forward.png"));
   private ImageIcon icPlay = new ImageIcon(loader.getResource("ffe/icons/play.png"));
   private ImageIcon icStop = new ImageIcon(loader.getResource("ffe/icons/stop.png"));
   private ImageIcon icStart = new ImageIcon(loader.getResource("ffe/icons/start.png"));
   private ImageIcon icBack = new ImageIcon(loader.getResource("ffe/icons/rewind.png"));

   //private ImageIcon icmeasure = new ImageIcon(loader.getResource("ffe/icons/measure.png"));
   //private ImageIcon ictrajectory = new ImageIcon(loader.getResource("ffe/icons/trajectory.png"));
   //private ImageIcon icsocket = new ImageIcon(loader.getResource("ffe/icons/socket.png"));

   private JMenuItem miPlay = new JMenuItem("Play", icPlay);
   private JMenuItem miStop = new JMenuItem("Stop", icStop);
   private JMenuItem miForward = new JMenuItem("Step Forward", icForward);
   private JMenuItem miBack = new JMenuItem("Step Back", icBack);
   private JMenuItem miReset = new JMenuItem("Reset", icStart);
   private JCheckBoxMenuItem jmiOscillate = new JCheckBoxMenuItem("Oscillate", icOscillate);
   private JMenuItem miFrame = new JMenuItem("Frame...", icBlank);
   private JMenuItem miSkip = new JMenuItem("Skip...", icBlank);
   private JMenuItem miSpeed = new JMenuItem("Speed...", icBlank);

   // Simulation Menu
   private JMenuItem miLocal = new JMenuItem("Connect to Local Job", icLocal);
   private JMenuItem miRemote = new JMenuItem("Connect to Remote Job", icNetwork);
   private JMenuItem miRelease = new JMenuItem("Release Job", icRelease);

   private JMenuItem miAddress = new JMenuItem("Set Remote Job Address...", icBlank);
   private JMenuItem miPort = new JMenuItem("Set Port...", icBlank);

   // Window Menu
   //private ImageIcon icchoose = new ImageIcon(loader.getResource("ffe/icons/view_choose.png"));
   private ImageIcon icReset = new ImageIcon(loader.getResource("ffe/icons/view_reset.png"));
   private ImageIcon icMenu = new ImageIcon(loader.getResource("ffe/icons/showmenu.png"));
   private ImageIcon icConsole = new ImageIcon(loader.getResource("ffe/icons/console.png"));

   //private ImageIcon icskin = new ImageIcon(loader.getResource("ffe/icons/looknfeel.png"));
   private ImageIcon icJava = new ImageIcon(loader.getResource("ffe/icons/java.png"));

   private JMenuItem miResize = new JMenuItem("Reset Panes", icReset);
   private JCheckBoxMenuItem cbSystems = new JCheckBoxMenuItem("Show Tree", icTree);
   private JCheckBoxMenuItem cbToolbar = new JCheckBoxMenuItem("Show Tool Bar", icMenu);
   private JCheckBoxMenuItem cbAxis = new JCheckBoxMenuItem("Show Global Axes", icHome);
   private JMenuItem miConsole = new JMenuItem("Show Console", icConsole);
   private JRadioButtonMenuItem rbJava = new JRadioButtonMenuItem("Java Metal Look & Feel", icJava);
   private JRadioButtonMenuItem rbNative = new JRadioButtonMenuItem("System Look & Feel", icBlank);
   private ButtonGroup bgSkin = new ButtonGroup();

   // Export Menu
   private ImageIcon icCamera = new ImageIcon(loader.getResource("ffe/icons/camera.png"));
   private JMenuItem miScreenCapture = new JMenuItem("Capture Graphics", icCamera);
   private JRadioButtonMenuItem rbJPEG = new JRadioButtonMenuItem("JPEG", icBlank);
   private JRadioButtonMenuItem rbPNG = new JRadioButtonMenuItem("PNG", icBlank);
   private ButtonGroup bgFormat = new ButtonGroup();

   // Help Menu
   private ImageIcon icHelp = new ImageIcon(loader.getResource("ffe/icons/help.png"));
   private JMenuItem miAbout = new JMenuItem("About...", icHelp);

   // Toolbar
   private JToolBar toolBar = new JToolBar("Toolbar", JToolBar.HORIZONTAL);

   // File Buttons
   private ImageIcon icOpen22 = new ImageIcon(loader.getResource("ffe/icons/fileopen22.png"));
   private ImageIcon icSaveas22 = new ImageIcon(loader.getResource("ffe/icons/filesaveas22.png"));
   private ImageIcon icClose22 = new ImageIcon(loader.getResource("ffe/icons/fileclose22.png"));
   private JButton jbOpen = new JButton(icOpen22);
   private JButton jbSave = new JButton(icSaveas22);
   private JButton jbClose = new JButton(icClose22);

   // Selection Buttons
   private ImageIcon icPick22 = new ImageIcon(loader.getResource("ffe/icons/pick22.png"));
   private ImageIcon icHighlight22 = new ImageIcon(loader.getResource("ffe/icons/highlight22.png"));
   private JButton jbPick = new JButton(icPick22);
   private JButton jbHighlight = new JButton(icHighlight22);

   // Window Buttons
   private ImageIcon icleft22 = new ImageIcon(loader.getResource("ffe/icons/left22.png"));
   private ImageIcon icright22 = new ImageIcon(loader.getResource("ffe/icons/right22.png"));
   private JButton jbleft = new JButton(icleft22);
   private JButton jbright = new JButton(icright22);

   // Graphics Buttons
   private ImageIcon ichome22 = new ImageIcon(loader.getResource("ffe/icons/gohome22.png"));
   private ImageIcon iczoomin22 = new ImageIcon(loader.getResource("ffe/icons/zoomin22.png"));
   private ImageIcon iczoomout22 = new ImageIcon(loader.getResource("ffe/icons/zoomout22.png"));
   private ImageIcon iccamera22 = new ImageIcon(loader.getResource("ffe/icons/camera22.png"));
   private JButton jbhome = new JButton(ichome22);
   private JButton jbzoomin = new JButton(iczoomin22);
   private JButton jbzoomout = new JButton(iczoomout22);
   private JButton jbcapture = new JButton(iccamera22);

   // Trajectory Buttons
   private ImageIcon icforward22 = new ImageIcon(loader.getResource("ffe/icons/forward22.png"));
   private ImageIcon icplay22 = new ImageIcon(loader.getResource("ffe/icons/play22.png"));
   private ImageIcon icstop22 = new ImageIcon(loader.getResource("ffe/icons/stop22.png"));
   private ImageIcon icrewind22 = new ImageIcon(loader.getResource("ffe/icons/rewind22.png"));
   private ImageIcon icstart22 = new ImageIcon(loader.getResource("ffe/icons/start22.png"));

   private JButton jbforward = new JButton(icforward22);
   private JButton jbplay = new JButton(icplay22);
   private JButton jbstop = new JButton(icstop22);
   private JButton jbrewind = new JButton(icstart22);
   private JButton jbback = new JButton(icrewind22);

   // Simulation Buttons
   private ImageIcon iclocal22 = new ImageIcon(loader.getResource("ffe/icons/local22.png"));
   private ImageIcon icremote22 = new ImageIcon(loader.getResource("ffe/icons/network22.png"));
   private ImageIcon icrelease22 = new ImageIcon(loader.getResource("ffe/icons/release22.png"));
   private JButton jblocal = new JButton(iclocal22);
   private JButton jbremote = new JButton(icremote22);
   private JButton jbrelease = new JButton(icrelease22);

   // Help
   private ImageIcon ichelp22 = new ImageIcon(loader.getResource("ffe/icons/help22.png"));
   private JButton jbhelp = new JButton(ichelp22);

   /*
    * Constructor
    * @param t Main application panel (for event listening)
    * @param g Main graphics panel (for event Listening)
    */
   public FMenuBar(MainPanel f){
      main = f;
      graphics = main.getGraphics3D();

      // Create the Tool Bar
      initToolBar();

      // Main Menubar
      addMenu(mFile, 'F');
      addMenu(mSelection, 'E');
      addMenu(mDisplay, 'D');
      addMenu(mColor, 'C');
      addMenu(mOptions, 'O');
      addMenu(mPicking, 'P');
      addMenu(mTrajectory, 'T');
      addMenu(mSimulation, 'S');
      addMenu(mExport, 'X');
      addMenu(mWindow, 'W');
      addMenu(mHelp, 'H');

      // File Menu
      addMenuItem(mFile, miOpen, 'O', KeyEvent.VK_O, main);
      //addMenuItem(mFile, miMerge, 'M', -1, main);
      addMenuItem(mFile, miSaveXYZ, 'S', KeyEvent.VK_S, main);
      addMenuItem(mFile, miClose, 'C', -1, main);
      addMenuItem(mFile, miCloseAll, 'A', -1, main);
      mFile.addSeparator();
      addMenuItem(mFile, miOpenPDB, 'D', KeyEvent.VK_D, main);
      addMenuItem(mFile, miLoadRestart, 'R', -1, main);
      addMenuItem(mFile, miLoadInduced, 'I', -1, main);
      mFile.addSeparator();
      addMenuItem(mFile, miExit, 'E', -1, main);

      // Selection Menu
      addMenuItem(mSelection, miSelectAll, 'A', KeyEvent.VK_A, main);
      addMenuItem(mSelection, miRestrict, 'R', KeyEvent.VK_R, graphics);
      mSelection.addSeparator();
      addMenuItem(mSelection, miHighlight, 'H', KeyEvent.VK_H, main);
      addMenuItem(mSelection, miSelectionColor, 'S', -1, graphics);
      mSelection.addSeparator();
      addMenuItem(mSelection, miLabelAtoms, 'O', -1, graphics);
      addMenuItem(mSelection, miLabelResidues, 'R', -1, graphics);
      addMenuItem(mSelection, miLabelSize, 'Z', -1, graphics);
      addMenuItem(mSelection, miLabelFontColor, 'C', -1, graphics);
      miHighlight.setSelected(false);
      miLabelAtoms.setSelected(false);
      miLabelResidues.setSelected(false);

      // Display Menu
      addMenuItem(mDisplay, miWireframe, 'W', -1, graphics);
      addMenuItem(mDisplay, miTube, 'T', -1, graphics);
      addMenuItem(mDisplay, miSpacefill, 'S', -1, graphics);
      addMenuItem(mDisplay, miBall, 'B', -1, graphics);
      addMenuItem(mDisplay, miInvisible, 'I', -1, graphics);
      addMenuItem(mDisplay, miRMIN, 'R', -1, graphics);
      mDisplay.addSeparator();
      addMenuItem(mDisplay, miFill, 'F', -1, graphics);
      addMenuItem(mDisplay, miPoints, 'P', -1, graphics);
      addMenuItem(mDisplay, miLines, 'L', -1, graphics);
      mDisplay.addSeparator();
      addMenuItem(mDisplay, miPreferences, 'R', -1, main);

      // Color Menu
      addMenuItem(mColor, miMonochrome, 'M', -1, graphics);
      addMenuItem(mColor, miAtomColor, 'C', -1, graphics);
      addMenuItem(mColor, miResidueColor, 'R', -1, graphics);
      addMenuItem(mColor, miStructureColor, 'S', -1, graphics);
      addMenuItem(mColor, miPolymerColor, 'M', -1, graphics);
      addMenuItem(mColor, miChargeColor, 'P', -1, graphics);
      addMenuItem(mColor, miVectorColor, 'V', -1, graphics);
      addMenuItem(mColor, miUsersColor, 'U', -1, graphics);
      mColor.addSeparator();
      addMenuItem(mColor, miApplyUserColor, 'A', -1, graphics);
      addMenuItem(mColor, miUserColor, 'C', -1, graphics);
      // Avoid conflicts with Atom picking
      miAtomColor.setActionCommand("byAtom");
      miResidueColor.setActionCommand("byResidue");
      miPolymerColor.setActionCommand("byPolymer");
      miVectorColor.setActionCommand("byVector");

      // Options Menu
      addMenuItem(mOptions, mActive, 'A', -1, null);
      addMenuItem(mOptions, mMode, 'D', -1, null);
      addMenuItem(mOptions, miZoom, 'Z', -1, graphics);
      addMenuItem(mOptions, miTranslation, 'O', -1, graphics);
      addMenuItem(mOptions, miRotation, 'N', -1, graphics);
      addMenuItem(mOptions, miView, 'V', KeyEvent.VK_V, graphics);
      //addMenuItem(mOptions, miOrigin, 'P', -1, graphics);
      //addMenuItem(mOptions, miResetOrigin, 'I', -1, graphics);

      mOptions.addSeparator();
      addMenuItem(mOptions, miHydrogens, 'H', -1, graphics);
      //     addMenuItem(mOptions, miVRML, 'V', -1, graphics);
      mOptions.addSeparator();

      /*
             addMenuItem(mOptions, miVectors, 'V', -1, graphics);
             addMenuItem(mOptions, mVector, 'Y', -1, null);
             addMenuItem(mOptions, mLength, 'L', -1, null);
             mOptions.addSeparator();
       */

      addMenuItem(mOptions, miFullScreen, 'F', KeyEvent.VK_F, main);
      addMenuItem(mOptions, miBackgroundColor, 'S', -1, graphics);

      // mVector.setIcon(icBlank);
      // mLength.setIcon(icBlank);
      mActive.setIcon(icBlank);
      mMode.setIcon(icBlank);

      addBGMenuItem(bgMode, mMode, miActive, 'A', KeyEvent.VK_A, graphics);
      addBGMenuItem(bgMode, mMode, miCanvas, 'S', KeyEvent.VK_M, graphics);
      miActive.setSelected(true);

      /*
             addMenuItem(mVector, miInduced, 'I', -1, graphics);
             addMenuItem(mVector, miForces, 'F', -1, graphics);
             addMenuItem(mVector, miVelocity, 'V', -1, graphics);
             addMenuItem(mVector, miAcceleration, 'A', -1, graphics);

             addMenuItem(mLength, miVUnit, 'U', -1, graphics);
             addMenuItem(mLength, miVRelative, 'R', -1, graphics);
             addMenuItem(mLength, miVAbsolute, 'A', -1, graphics);
       */

      addMenuItem(mActive, miRemoveRotation, 'A', -1, graphics);
      addMenuItem(mActive, miRemoveTranslation, 'L', -1, graphics);
      addMenuItem(mActive, miRemoveRotTrans, 'D', -1, graphics);
      addMenuItem(mActive, miRotateOnCOM, 'C', -1, graphics);
      addMenuItem(mActive, miRotateOnPick, 'P', -1, graphics);

      // Picking Menu
      addMenuItem(mPicking, miPicking, 'G', KeyEvent.VK_P, graphics);
      miPicking.setSelected(false);
      mPicking.addSeparator();

      addBGMenuItem(bgLevel, mPicking, rbAtom, 'A', -1, graphics);
      addBGMenuItem(bgLevel, mPicking, rbBond, 'B', -1, graphics);
      addBGMenuItem(bgLevel, mPicking, rbAngle, 'N', -1, graphics);
      addBGMenuItem(bgLevel, mPicking, rbDihedral, 'D', -1, graphics);
      addBGMenuItem(bgLevel, mPicking, rbResidue, 'R', -1, graphics);
      addBGMenuItem(bgLevel, mPicking, rbPolymer, 'P', -1, graphics);
      addBGMenuItem(bgLevel, mPicking, rbMolecule, 'M', -1, graphics);
      addBGMenuItem(bgLevel, mPicking, rbSystem, 'S', -1, graphics);
      mPicking.addSeparator();

      addBGMenuItem(bgLevel, mPicking, rbMeasureDistance, 'I', -1, graphics);
      addBGMenuItem(bgLevel, mPicking, rbMeasureAngle, 'L', -1, graphics);
      addBGMenuItem(bgLevel, mPicking, rbMeasureDihedral, 'H', -1, graphics);
      rbAtom.setSelected(true);
      mPicking.addSeparator();
      addMenuItem(mPicking, miPickColor, 'S', -1, graphics);

      // Trajectory Menu
      addMenuItem(mTrajectory, jmiOscillate, 'O', -1, main);
      jmiOscillate.setSelected(false);
      addMenuItem(mTrajectory, miFrame, 'A', -1, main);
      addMenuItem(mTrajectory, miSpeed, 'E', -1, main);
      addMenuItem(mTrajectory, miSkip, 'K', -1, main);
      mTrajectory.addSeparator();

      addMenuItem(mTrajectory, miPlay, 'P', -1, main);
      addMenuItem(mTrajectory, miStop, 'S', -1, main);
      addMenuItem(mTrajectory, miForward, 'F', -1, main);
      addMenuItem(mTrajectory, miBack, 'B', -1, main);
      addMenuItem(mTrajectory, miReset, 'R', -1, main);

      // Simulation Menu
      addMenuItem(mSimulation, miRelease, 'E', -1, main);
      addMenuItem(mSimulation, miLocal, 'L', KeyEvent.VK_L, main);
      mSimulation.addSeparator();

      addMenuItem(mSimulation, miRemote, 'R', -1, main);
      addMenuItem(mSimulation, miAddress, 'A', -1, main);
      mSimulation.addSeparator();

      addMenuItem(mSimulation, miPort, 'P', -1, main);
      miAddress.setIcon(icBlank);

      // Export
      addMenuItem(mExport, miScreenCapture, 'C', KeyEvent.VK_G, graphics);
      mExport.addSeparator();
      addBGMenuItem(bgFormat, mExport, rbPNG, 'P', -1, graphics);
      addBGMenuItem(bgFormat, mExport, rbJPEG, 'J', -1, graphics);
      rbPNG.setSelected(true);

      // Window Options
      addMenuItem(mWindow, miResize, 'R', -1, main);
      mWindow.addSeparator();

      addMenuItem(mWindow, cbSystems, 'T', -1, main);
      addMenuItem(mWindow, cbToolbar, 'B', -1, main);
      addMenuItem(mWindow, cbAxis, 'C', -1, main);
      addMenuItem(mWindow, miConsole, 'L', -1, main);
      cbAxis.setSelected(true);
      mWindow.addSeparator();

      addBGMenuItem(bgSkin, mWindow, rbJava, 'J', -1, main);
      addBGMenuItem(bgSkin, mWindow, rbNative, 'S', -1, main);
      rbJava.setSelected(true);

      String osname = System.getProperty("os.name", null);
      if (osname != null){
         osname = osname.toLowerCase();
         if (osname.startsWith("l")){
            ImageIcon ictux = new ImageIcon(loader.getResource("ffe/icons/tux.png"));
            rbNative.setText("Motif Look & Feel");
            rbNative.setMnemonic('M');
            rbNative.setIcon(ictux);
         } else if (osname.startsWith("w")){
            ImageIcon icwindows = new ImageIcon(loader.getResource("ffe/icons/win_apps.png"));
            rbNative.setText("Windows Look & Feel");
            rbNative.setMnemonic('W');
            rbNative.setIcon(icwindows);
         } else if (osname.startsWith("m")){
            ImageIcon icmac = new ImageIcon(loader.getResource("ffe/icons/mac.png"));
            rbNative.setText("MacOSX Look & Feel");
            rbNative.setMnemonic('M');
            rbNative.setIcon(icmac);
            rbNative.setSelected(true);
         }
         rbNative.setActionCommand("Native");
      }

      // Help/About Menu
      addMenuItem(mHelp, miAbout, 'A', -1, main);
   }

   private void addMenu(JMenu mi, char mnem){
      add(mi);
      if (mnem != '.'){
         mi.setMnemonic(mnem);
      }
   }

   private static int keyMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

   private void addMenuItem(JMenu menu, JMenuItem mi, char mnem, int acc, ActionListener al){
      menu.add(mi);
      if (mnem != '.'){
         mi.setMnemonic(mnem);
      }
      if (acc != -1){
         mi.setAccelerator(KeyStroke.getKeyStroke(acc, keyMask));
      }
      if (al != null){
         mi.addActionListener(al);
      }
   }

   private void addBGMenuItem(ButtonGroup bg, JMenu menu, JMenuItem mi, char mnem, int acc,
      ActionListener al){
      bg.add(mi);
      addMenuItem(menu, mi, mnem, acc, al);
   }

   /**
    *  Initialize the tool bar short cut buttons
    */
   private void initToolBar(){

      // File Buttons
      Insets insets = jbOpen.getInsets();
      insets.left = 2;
      insets.right = 2;
      insets.bottom = 2;
      insets.top = 2;
      jbOpen.setActionCommand("Open...");
      jbOpen.addActionListener(main);
      jbOpen.setToolTipText("Open a *.XYZ File");
      jbOpen.setMargin(insets);
      toolBar.add(jbOpen);
      jbSave.setActionCommand("Save as...");
      jbSave.addActionListener(main);
      jbSave.setToolTipText("Save the Active System");
      jbSave.setMargin(insets);
      toolBar.add(jbSave);
      jbClose.setActionCommand("Close");
      jbClose.addActionListener(main);
      jbClose.setToolTipText("Close the Active System");
      jbClose.setMargin(insets);
      toolBar.add(jbClose);
      toolBar.addSeparator();

      // Selection Buttons
      jbHighlight.setActionCommand("Highlight Selections");
      jbHighlight.addActionListener(main);
      jbHighlight.setToolTipText("Toggle Highlighting");
      jbHighlight.setMargin(insets);
      toolBar.add(jbHighlight);
      jbPick.setActionCommand("Graphics Picking");
      jbPick.addActionListener(graphics);
      jbPick.setToolTipText("Toggle Graphics Picking");
      jbPick.setMargin(insets);
      toolBar.add(jbPick);

      // Scene Buttons
      jbhome.setActionCommand("Center Coordinate System");
      jbhome.addActionListener(main);
      jbhome.setToolTipText("Center the Coordinate System");
      jbhome.setMargin(insets);
      toolBar.add(jbhome);
      jbzoomout.setActionCommand("Zoom Out");
      jbzoomout.addActionListener(graphics);
      jbzoomout.setToolTipText("Zoom Out");
      jbzoomout.setMargin(insets);
      toolBar.add(jbzoomout);
      jbzoomin.setActionCommand("Zoom In");
      jbzoomin.addActionListener(graphics);
      jbzoomin.setToolTipText("Zoom In");
      jbzoomin.setMargin(insets);
      toolBar.add(jbzoomin);
      jbcapture.setActionCommand("Capture Graphics");
      jbcapture.setToolTipText("Capture the Graphics Window to a Image File");
      jbcapture.addActionListener(graphics);
      jbcapture.setMargin(insets);
      toolBar.add(jbcapture);
      toolBar.addSeparator();

      jbleft.setActionCommand("Split Left");
      jbleft.addActionListener(main);
      jbleft.setToolTipText("Expand the Graphics Window");
      jbleft.setMargin(insets);
      toolBar.add(jbleft);
      jbright.setActionCommand("Split Right");
      jbright.addActionListener(main);
      jbright.setToolTipText("Shrink the Graphics Window");
      jbright.setMargin(insets);
      toolBar.add(jbright);
      toolBar.addSeparator();

      // Trajectory Buttons
      jbforward.setActionCommand("Step Forward");
      jbforward.addActionListener(main);
      jbforward.setToolTipText("Advance One Frame");
      jbforward.setMargin(insets);
      toolBar.add(jbforward);
      jbplay.setActionCommand("Play");
      jbplay.addActionListener(main);
      jbplay.setToolTipText("Play the Active Trajectory");
      jbplay.setMargin(insets);
      toolBar.add(jbplay);
      jbstop.setActionCommand("Stop");
      jbstop.addActionListener(main);
      jbstop.setToolTipText("Stop the Active Trajectory");
      jbstop.setMargin(insets);
      toolBar.add(jbstop);
      jbback.setActionCommand("Step Back");
      jbback.addActionListener(main);
      jbback.setToolTipText("Back Up One Frame");
      jbback.setMargin(insets);
      toolBar.add(jbback);
      jbrewind.setActionCommand("Reset");
      jbrewind.addActionListener(main);
      jbrewind.setToolTipText("Reset the Active Trajectory to Frame 1");
      jbrewind.setMargin(insets);
      toolBar.add(jbrewind);
      toolBar.addSeparator();

      // Simulation Buttons
      jblocal.setActionCommand("Connect to Local Job");
      jblocal.setToolTipText("Connect to a Local TINKER Simulation");
      jblocal.addActionListener(main);
      jblocal.setMargin(insets);
      toolBar.add(jblocal);
      jbremote.setActionCommand("Connect to Remote Job");
      jbremote.setToolTipText("Connect to a Remote TINKER Simulation");
      jbremote.addActionListener(main);
      jbremote.setMargin(insets);
      toolBar.add(jbremote);
      jbrelease.setActionCommand("Release Job");
      jbrelease.setToolTipText("Release the current Simulation");
      jbrelease.addActionListener(main);
      jbrelease.setMargin(insets);
      toolBar.add(jbrelease);
      toolBar.addSeparator();

      // Help
      jbhelp.setActionCommand("About...");
      jbhelp.addActionListener(main);
      jbhelp.setToolTipText("TINKER/FFE Logo and Contact Information");
      jbhelp.setMargin(insets);
      toolBar.add(jbhelp);
      // Toolbar Configuration
      toolBar.setBorderPainted(true);
      toolBar.setRollover(true);
   }

   /**
    * Get a reference the tool bar
    * @return Force Field Explorer ToolBar
    */
   public JToolBar getToolBar(){
      return toolBar;
   }

   public boolean isMenuShowing(){
      return cbToolbar.isSelected();
   }

   public void setMenuShowing(boolean b){
      cbToolbar.setSelected(b);
   }

   public void menuClick(){
      cbToolbar.doClick();
   }

   public void setAxisShowing(boolean b){
      cbAxis.setSelected(b);
   }

   public boolean isAxisShowing(){
      return cbAxis.isSelected();
   }

   public boolean isSystemShowing(){
      return cbSystems.isSelected();
   }

   public void setSystemShowing(boolean b){
      cbSystems.setSelected(b);
   }

   public void systemClick(){
      cbSystems.doClick();
   }

   /**
    * Toggle connection status
    */
   public void setConnect(boolean b){
      miLocal.setEnabled(b);
      miRemote.setEnabled(b);
      miRelease.setEnabled(!b);
   }

   public boolean getPicking(){
      return miPicking.isSelected();
   }

   public void setPickBehavior(boolean pick){
      miPicking.setSelected(pick);
   }

   public boolean isPickingActive(){
      return miPicking.isSelected();
   }

   public void setPickLevel(String arg){
      if (arg.equals("Atom")){
         rbAtom.doClick();
      } else if (arg.equals("Bond")){
         rbBond.doClick();
      } else if (arg.equals("Angle")){
         rbAngle.doClick();
      } else if (arg.equals("Dihedral")){
         rbDihedral.doClick();
      } else if (arg.equals("Residue")){
         rbResidue.doClick();
      } else if (arg.equals("Polymer")){
         rbPolymer.doClick();
      } else if (arg.equals("Molecule")){
         rbMolecule.doClick();
      } else if (arg.equals("System")){
         rbSystem.doClick();
      } else if (arg.equals("Measure Distance")){
         rbMeasureDistance.doClick();
      } else if (arg.equals("Measure Angle")){
         rbMeasureAngle.doClick();
      } else if (arg.equals("Measure Dihedral")){
         rbMeasureDihedral.doClick();
      }
      return;
   }

   public boolean getHighlighting(){
      return miHighlight.isSelected();
   }

   public void setHighlighting(boolean h){
      miHighlight.setSelected(h);
   }

   public void setAtomLabels(boolean b){
      miLabelAtoms.setSelected(b);
   }

   public void setResidueLabels(boolean b){
      miLabelResidues.setSelected(b);
   }

   public void setMouseMode(int m){
      if (m == Graphics3D.ACTIVE){
         miActive.doClick();
      } else{
         miCanvas.doClick();
      }
   }

   public int getMouseMode(){
      if (miActive.isSelected()){
         return Graphics3D.ACTIVE;
      } else{
         return Graphics3D.SCREEN;
      }
   }
}
