/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.panels;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * The FullScreenWindow class controls full screen graphics.
 */
public class FullScreenWindow
   extends Window
   implements KeyListener{

   JFrame mainFrame;
   JPanel graphicsPanel;
   Graphics3D graphics;
   Dimension screenSize;
   //BufferStrategy oldBufferStrategy;

   boolean fullScreen = false;

   public FullScreenWindow(JFrame frame, JPanel gp, Graphics3D g){
      super(frame);
      mainFrame = frame;
      graphicsPanel = gp;
      graphics = g;
      Toolkit toolkit = getToolkit();
      screenSize = toolkit.getScreenSize();
      setSize(screenSize);
   }

   public void toggleFullScreen(){
      if (fullScreen){
         exitFullScreen();
      } else{
         enterFullScreen();
      }
   }

   public void enterFullScreen(){
      if (fullScreen){
         return;
      }
      //oldBufferStrategy = graphics.getBufferStrategy();
      graphics.addKeyListener(this);
      mainFrame.addKeyListener(this);
      setSize(screenSize);
      graphicsPanel.remove(graphics);
      //getContentPane().add(graphics);
      add(graphics);
      show();
      toFront();
      /* if (GraphicsEnvironment.getLocalGraphicsEnvironment().
          getDefaultScreenDevice().isFullScreenSupported()) {
         try {
            GraphicsEnvironment.getLocalGraphicsEnvironment().
               getDefaultScreenDevice().setFullScreenWindow(this);
            /*
              GraphicsConfiguration bufs[] = GraphicsEnvironment.
                                                     getLocalGraphicsEnvironment().
                                                     getDefaultScreenDevice().
                                                     getConfigurations();
                                   for (int i = 0; i < bufs.length; i++) {
                         BufferCapabilities buf = bufs[i].getBufferCapabilities();
                         if (buf.isPageFlipping()){
                            graphics.createBufferStrategy(2,buf);
                            System.out.println("Page Flipping Mode...");
                            break;
                         }
                                   }

                   }
                   catch (Exception e) {
                   }
                       }
        */fullScreen = true;
   }

   public void exitFullScreen(){
      if (!fullScreen){
         return;
      }
      graphics.removeKeyListener(this);
      mainFrame.removeKeyListener(this);
      /*
             if (GraphicsEnvironment.getLocalGraphicsEnvironment().
          getDefaultScreenDevice().isFullScreenSupported()) {
         try {
            //graphics.createBufferStrategy(2);
            GraphicsEnvironment.getLocalGraphicsEnvironment().
               getDefaultScreenDevice().setFullScreenWindow(null);
         }
         catch (Exception e) {
         }
             }
       */remove(graphics);
      graphicsPanel.add(graphics, BorderLayout.CENTER);
      hide();
      mainFrame.toFront();
      fullScreen = false;
   }

   public void keyPressed(KeyEvent evt){
      if (evt.getKeyCode() == evt.VK_ESCAPE){
         exitFullScreen();
      } else if (evt.getKeyChar() == 'e' || evt.getKeyChar() == 'x'){
         exitFullScreen();
      }
   }

   public void keyReleased(KeyEvent evt){
   }

   public void keyTyped(KeyEvent evt){
   }
}
