/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.panels;

import javax.vecmath.Color3f;

import java.awt.Color;

/**
 * The GraphicsContext class defines constants related to rendering modes.
 */

public final class GraphicsContext{

   public int viewtype = WIREFRAME;
   public int colortype = byATOM;
   public static int detail = 3;
   public static double radius = 1.0d;
   public static int bondwidth = 3;
   public static Color3f pickingColor = new Color3f(Color.MAGENTA);
   public static Color3f selectionColor = new Color3f(Color.GREEN);
   public static boolean highlightSelections = false;
   public static Color3f userColor = new Color3f(Color.WHITE);
   public static boolean labelAtoms = false;
   public static boolean labelResidues = false;

   public GraphicsContext(){}

   public GraphicsContext(int v, int c){
      viewtype = v;
      colortype = c;
   }

   // Create colors that will be used frequently
   public static final Color3f blackColor = new Color3f(Color.black);
   public static final Color3f whiteColor = new Color3f(Color.white);
   public static final Color3f helixColor = new Color3f(Color.magenta);
   public static final Color3f sheetColor = new Color3f(Color.orange);
   public static final Color3f turnColor = new Color3f(Color.blue);
   public static final Color3f dipoleColor = new Color3f(Color.blue);
   public static final Color3f dangleColor = new Color3f(Color.orange);
   public static final Color3f nullColor = new Color3f(Color.darkGray);
   public static final Color3f lightGray = new Color3f(Color.lightGray);
   public static final Color3f greenColor = new Color3f(Color.GREEN);
   public static final Color bgColor = Color.black;
   public static final Color3f backgroundColor = new Color3f(bgColor);

   public static final Color3f toAtomColor(String s){
      String c = s.toLowerCase();
      if (c.startsWith("h")){
         return whiteColor;
      }
      if (c.startsWith("c")){
         return lightGray;
      }
      if (c.startsWith("n")){
         return dipoleColor;
      }
      if (c.startsWith("o")){
         return helixColor;
      }
      if (c.startsWith("p")){
         return greenColor;
      }
      if (c.startsWith("s")){
         return turnColor;
      }
      return nullColor;
   }

   // Viewing Models
   public static final int WIREFRAME = 0;
   public static final int BALLSTICK = 1;
   public static final int SPACEFILL = 2;
   public static final int RMIN = 3;
   public static final int TUBE = 4;
   public static final int POINTS = 5;
   public static final int LINES = 6;
   public static final int FILL = 7;
   public static final int RIBBON = 8;
   public static final int INVISIBLE = 9;
   public static final int RESTRICT = 10;
   public static final int DETAIL = 11;
   public static final int HYDROGENS = 12;
   public static final int INDUCED = 13;
   public static final int PERMANENT = 14;
   public static final int FORCE = 15;
   public static final int VELOCITY = 16;
   public static final int ACCELERATION = 17;
   public static final int DESTROY = 18;
   public static final int UNIT = 19;
   public static final int RELATIVE = 20;
   public static final int ABSOLUTE = 21;
   public static final int VECTORS = 22;
   public static final int VRML = 23;

   public static final int toView(String s){
      if (s.equalsIgnoreCase("WIREFRAME")){
         return WIREFRAME;
      }
      if (s.equalsIgnoreCase("BALL & STICK")){
         return BALLSTICK;
      }
      if (s.equalsIgnoreCase("SPACEFILL")){
         return SPACEFILL;
      }
      if (s.equalsIgnoreCase("TUBE")){
         return TUBE;
      }
      if (s.equalsIgnoreCase("RIBBON")){
         return RIBBON;
      }
      if (s.equalsIgnoreCase("INVISIBLE")){
         return INVISIBLE;
      }
      if (s.equalsIgnoreCase("RESTRICT") || s.equalsIgnoreCase("RESTRICT TO SELECTIONS")){
         return RESTRICT;
      }
      if (s.equalsIgnoreCase("DETAIL")){
         return DETAIL;
      }
      if (s.equalsIgnoreCase("RMIN")){
         return RMIN;
      }
      if (s.equalsIgnoreCase("HYDROGENS") || s.equalsIgnoreCase("TOGGLE HYDROGENS")){
         return HYDROGENS;
      }
      if (s.equalsIgnoreCase("INDUCED") || s.equalsIgnoreCase("INDUCED DIPOLES") ||
         s.equalsIgnoreCase("INDUCED DIPOLE")){
         return INDUCED;
      }
      if (s.equalsIgnoreCase("PERMANENT") || s.equalsIgnoreCase("PERMANENT DIPOLES")){
         return PERMANENT;
      }
      if (s.equalsIgnoreCase("FORCE") || s.equalsIgnoreCase("FORCES")){
         return FORCE;
      }
      if (s.equalsIgnoreCase("VELOCITY") || s.equalsIgnoreCase("VELOCITIES")){
         return VELOCITY;
      }
      if (s.equalsIgnoreCase("ACCELERATION") || s.equalsIgnoreCase("ACCELERATIONS")){
         return ACCELERATION;
      }
      if (s.equalsIgnoreCase("VECTORS") || s.equalsIgnoreCase("TOGGLE VECTORS")){
         return VECTORS;
      }
      if (s.equalsIgnoreCase("FILL")){
         return FILL;
      }
      if (s.equalsIgnoreCase("POINTS")){
         return POINTS;
      }
      if (s.equalsIgnoreCase("LINES")){
         return LINES;
      }
      if (s.equalsIgnoreCase("UNIT")){
         return UNIT;
      }
      if (s.equalsIgnoreCase("RELATIVE")){
         return RELATIVE;
      }
      if (s.equalsIgnoreCase("ABSOLUTE")){
         return ABSOLUTE;
      }
      if (s.equalsIgnoreCase("VRML") || s.equalsIgnoreCase("TOGGLE VRML")){
         return VRML;
      }
      return -1;
   }

   public static final String toViewString(int c){
      if (c == 0){
         return "Wireframe".intern();
      } else if (c == 1){
         return "Ball & Stick".intern();
      } else if (c == 2){
         return "Spacefilling".intern();
      } else if (c == 3){
         return "RMIN".intern();
      } else if (c == 4){
         return "Tube".intern();
      } else if (c == 5){
         return "Points".intern();
      } else if (c == 6){
         return "Lines".intern();
      } else if (c == 7){
         return "Fill".intern();
      } else if (c == 8){
         return "Ribbons".intern();
      } else if (c == 9){
         return "Invisible".intern();
      } else if (c == 10){
         return "Restrict to Selections".intern();
      } else if (c == 11){
         return "Detail".intern();
      } else if (c == 12){
         return "Toggle Hydrogens".intern();
      } else if (c == 13){
         return "Induced Dipoles".intern();
      } else if (c == 14){
         return "Permanent".intern();
      } else if (c == 15){
         return "Force".intern();
      } else if (c == 16){
         return "Velocity".intern();
      } else if (c == 17){
         return "Acceleration".intern();
      } else if (c == 18){
         return "Destroy".intern();
      } else if (c == 19){
         return "Unit Magnitude".intern();
      } else if (c == 20){
         return "Relative Magnitude".intern();
      } else if (c == 21){
         return "Absolute Magnitude".intern();
      } else if (c == 22){
         return "Toggle Vectors".intern();
      } else if (c == 23){
         return "Toggle VRML".intern();
      }
      return null;
   }

   // Coloring Models
   public static final int byATOM = 0;
   public static final int byGROUP = 1;
   public static final int byMOLECULE = 2;
   public static final int byPICK = 3;
   public static final int byCHARGE = 4;
   public static final int byINDUCED = 5;
   public static final int byVELOCITY = 6;
   public static final int byACCELERATION = 7;
   public static final int byFORCE = 8;
   public static final int byVECTOR = 9;
   public static final int byUSER = 10;
   public static final int bySELECT = 11;
   public static final int setUSER = 12;
   public static final int byMONOCHROME = 13;
   public static final int byPOLYMER = 14;
   public static final int byRESIDUE = 15;
   public static final int bySTRUCTURE = 16;
   public static final int byREVERT = 17;

   //public static final int byTEMPERATURE = 17;

   public static final int toColor(String s){
      if (s.equalsIgnoreCase("ATOM") || s.equalsIgnoreCase("CPK") || s.equalsIgnoreCase("BYATOM")){
         return byATOM;
      }
      if (s.equalsIgnoreCase("RESIDUE") || s.equalsIgnoreCase("BYRESIDUE")){
         return byRESIDUE;
      }
      if (s.equalsIgnoreCase("POLYMER") || s.equalsIgnoreCase("BYPOLYMER")){
         return byPOLYMER;
      }
      if (s.equalsIgnoreCase("PICK") || s.equalsIgnoreCase("BYPICK")){
         return byPICK;
      }
      if (s.equalsIgnoreCase("PARTIAL CHARGE") || s.equalsIgnoreCase("CHARGE") ||
         s.equalsIgnoreCase("BYCHARGE")){
         return byCHARGE;
      }
      if (s.equalsIgnoreCase("BYVELOCITY")){
         return byVELOCITY;
      }
      if (s.equalsIgnoreCase("BYACCELERATION")){
         return byACCELERATION;
      }
      if (s.equalsIgnoreCase("BYFORCE")){
         return byFORCE;
      }
      if (s.equalsIgnoreCase("BYINDUCED")){
         return byINDUCED;
      }
      if (s.equalsIgnoreCase("BYVECTOR")){
         return byVECTOR;
      }
      if (s.equalsIgnoreCase("USER") || s.equalsIgnoreCase("USER COLOR")){
         return byUSER;
      }
      if (s.equalsIgnoreCase("SELECT")){
         return bySELECT;
      }
      if (s.equalsIgnoreCase("SET USER") || s.equalsIgnoreCase("APPLY USER COLOR TO SELECTIONS")){
         return setUSER;
      }
      if (s.equalsIgnoreCase("MONOCHROME")){
         return byMONOCHROME;
      }
      if (s.equalsIgnoreCase("STRUCTURE")){
         return bySTRUCTURE;
      }
      return -1;
   }

   public static final String toColorString(int c){
      if (c == 0){
         return "Atom".intern();
      } else if (c == 1){
         return "Group".intern();
      } else if (c == 2){
         return "Molecule".intern();
      } else if (c == 3){
         return "Graphics Picking".intern();
      } else if (c == 4){
         return "Charge".intern();
      } else if (c == 5){
         return "Induced".intern();
      } else if (c == 6){
         return "Velocity".intern();
      } else if (c == 7){
         return "Acceleration".intern();
      } else if (c == 8){
         return "Force".intern();
      } else if (c == 9){
         return "Vector".intern();
      } else if (c == 10){
         return "User Color".intern();
      } else if (c == 11){
         return "Selection".intern();
      } else if (c == 12){
         return "Set User Color".intern();
      } else if (c == 13){
         return "Monochrome".intern();
      } else if (c == 14){
         return "Polymer".intern();
      } else if (c == 15){
         return "Residue".intern();
      } else if (c == 16){
         return "Stucture".intern();
      } else if (c == 17){
         return "Previous".intern();
      }
      return " ";
   }
}
