/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Zhiguang Gao, Michael J. Schnieders
 * @version 4.2
 */

package ffe.panels;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import ffe.lang.*;

/**
 * The GraphicsDialog class allows users to select graphics preferences.
 */
public class GraphicsDialog
   extends JDialog
   implements ActionListener{

   private Root root;
   private GridBagConstraints constraints;

   /**
    * Contructor
    * @param frame Parent frame
    * @param r Data structure root
    * @param g Graphics Canvas
    * @param gr Panel that contains the Graphics Canvas
    */
   public GraphicsDialog(Frame frame, Root r){
      super(frame, "", true);
      root = r;
      setTitle("Graphics Preferences");
      setSize(400, 200);
      setResizable(false);
      getContentPane().setLayout(new GridBagLayout());
      constraints = new GridBagConstraints();
      constraints.weighty = 100;
      constraints.gridheight = 1;
      constraints.gridwidth = 2;
      constraints.ipadx = 5;
      constraints.ipady = 5;
      constraints.gridx = 0;
      constraints.gridy = 0;
      constraints.insets = new Insets(5, 5, 5, 5);

      // Slider for radius
      JSlider radius = new JSlider(0, 200, 1);
      radius.setMajorTickSpacing(50);
      radius.setMinorTickSpacing(10);
      radius.setPaintLabels(true);
      radius.setPaintTicks(true);
      radius.setValue((int) (GraphicsContext.radius * 100));

      Hashtable labelTable = new Hashtable();
      labelTable.put(new Integer(1), new JLabel("0%"));
      labelTable.put(new Integer(50), new JLabel("50%"));
      labelTable.put(new Integer(100), new JLabel("100%"));
      labelTable.put(new Integer(150), new JLabel("150%"));
      labelTable.put(new Integer(200), new JLabel("200%"));
      radius.setLabelTable(labelTable);
      addSlider(radius, " Radius", 1);

      /*
               // Slider for InduceDipole
               JSlider dipole = new JSlider(0, 200, 5);
               dipole.setMajorTickSpacing(50);
               dipole.setMinorTickSpacing(10);
               dipole.setPaintLabels(true);
               dipole.setPaintTicks(true);
               dipole.setLabelTable(labelTable);
               dipole.setValue( (int) (Graphics.getInduced() * 100));
               addSlider(dipole, " Induced Dipole", 4);
       */

      // Slider for bondwidth
      JSlider bondwidth = new JSlider(1, 5, 5);
      bondwidth.setMajorTickSpacing(1);
      bondwidth.setMinorTickSpacing(1);
      bondwidth.setPaintLabels(true);
      bondwidth.setPaintTicks(true);
      labelTable = new Hashtable();
      labelTable.put(new Integer(1), new JLabel("1"));
      labelTable.put(new Integer(3), new JLabel("3"));
      labelTable.put(new Integer(5), new JLabel("5"));
      bondwidth.setLabelTable(labelTable);
      bondwidth.setValue(GraphicsContext.bondwidth);
      addSlider(bondwidth, " Wireframe Thickness", 3);

      // Slider for detail
      JSlider detail = new JSlider(1, 5, 2);
      detail.setMajorTickSpacing(1);
      detail.setMinorTickSpacing(1);
      detail.setPaintLabels(true);
      detail.setPaintTicks(true);
      detail.setValue((int) GraphicsContext.detail);

      labelTable = new Hashtable();
      labelTable.put(new Integer(1), new JLabel("Fast"));
      labelTable.put(new Integer(5), new JLabel("Pretty"));
      detail.setLabelTable(labelTable);
      addSlider(detail, " Detail", 2);

      constraints.gridwidth = 1;
      JButton jb = new JButton("Apply");
      jb.addActionListener(this);
      getContentPane().add(jb, constraints);
      JButton jbclose = new JButton("Close");
      jbclose.addActionListener(this);
      constraints.gridx++;
      getContentPane().add(jbclose, constraints);

      pack();
      Dimension dim = getToolkit().getScreenSize();
      Dimension ddim = getSize();
      setLocation((dim.width - ddim.width) / 2, (dim.height - ddim.height) / 2);
   }

   public void actionPerformed(ActionEvent e){
      if (e.getActionCommand().equalsIgnoreCase("Apply")){
         FSystem s;
         for (Enumeration ee = root.children(); ee.hasMoreElements(); ){
            s = (FSystem) ee.nextElement();
            if (s != null){
               s.switchView(GraphicsContext.DETAIL);
            }
         }
      } else if (e.getActionCommand().equalsIgnoreCase("Close")){
         FSystem s;
         for (Enumeration ee = root.children(); ee.hasMoreElements(); ){
            s = (FSystem) ee.nextElement();
            if (s != null){
               s.switchView(GraphicsContext.DETAIL);
            }
         }
         dispose();
      }
   }

   public void addSlider(JSlider s, String description, final int it){
      final EtchedBorder eb = new EtchedBorder(EtchedBorder.LOWERED);
      s.setSnapToTicks(true);
      s.setBorder(new TitledBorder(eb, description));
      s.addChangeListener(new ChangeListener(){
         public void stateChanged(ChangeEvent e){
            JSlider source = (JSlider) e.getSource();
            if (source.getValueIsAdjusting()){
               return;
            }
            int value = source.getValue();
            if (it == 1){
               if (value < 1){
                  return;
               }
               GraphicsContext.radius = value / 100.0d;
            } else if (it == 2){
               GraphicsContext.detail = value;
            } else if (it == 3){
               GraphicsContext.bondwidth = value;
            }
         }
      });

      // add three components into the next row
      constraints.gridx = 0;
      constraints.anchor = GridBagConstraints.WEST;
      constraints.fill = GridBagConstraints.NONE;
      getContentPane().add(s, constraints);
      constraints.gridy++;
   }
}
