/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.panels;

import java.awt.Container;
import java.awt.event.*;
import javax.swing.*;

/**
 * The KeyFileEditor class is a wrapper for the KeywordPanel to create
 * a stand alone TINKER Key File Editor, but it needs updating.
 */
public final class KeyFileEditor
   extends JFrame{

   KeywordPanel tkp;

   public KeyFileEditor(){

      try{
         UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
      } catch (Exception e){
         System.out.println("Can't set look and feel: " + e);
      }

      tkp = new KeywordPanel(null);
      Container contentPane = getContentPane();
      contentPane.add(tkp);
      setTitle("Key File Editor");
      setSize(800, 800);

      addWindowListener(new WindowAdapter(){
         public void windowClosing(WindowEvent e){
            close();
            System.exit(0);
         }
      });

   }

   private void close(){
      if (tkp.isFileOpen()){
         int option = JOptionPane.showConfirmDialog(this, "Save Changes First",
                      "Closing Key Editor", JOptionPane.YES_NO_OPTION,
                      JOptionPane.INFORMATION_MESSAGE);
         if (option == JOptionPane.YES_OPTION){
            tkp.keySave(null);
         }
      }
   }

   public static void main(String[] args){
      KeyFileEditor editor = new KeyFileEditor();
      editor.show();

   }
}
