/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.parsers;

import java.io.*;
import java.util.*;

import ffe.lang.*;

/**
 * The DYNFilter class parses TINKER Restart (*.DYN) files.
 */

public class DYNFilter{
   FSystem fsystem;
   File file;

   public DYNFilter(FSystem s, File f){
      fsystem = s;
      file = f;
   }

   public boolean read(){

      if (!file.exists() || !file.canRead()){
         return false;

      }
      try{
         FileReader fr = new FileReader(file);
         BufferedReader br = new BufferedReader(fr);

         br.readLine();

         String data = br.readLine().trim();
         String tokens[] = data.split(" +");
         if (tokens.length == 0){
            return false;
         }
         int numatoms = Integer.parseInt(tokens[0]);
         if (numatoms != fsystem.getAtomList().size()){
            return false;
         }

         br.readLine();
         data = br.readLine().trim();
         tokens = data.split(" +");
         if (tokens.length != 3){
            return false;
         }
         double d[] = new double[3];
         d[0] = Double.parseDouble(tokens[0]);
         d[1] = Double.parseDouble(tokens[1]);
         d[2] = Double.parseDouble(tokens[2]);
         fsystem.setBox(d);

         data = br.readLine().trim();
         tokens = data.split(" +");
         if (tokens.length != 3){
            return false;
         }
         d[0] = Double.parseDouble(tokens[0]);
         d[1] = Double.parseDouble(tokens[1]);
         d[2] = Double.parseDouble(tokens[2]);
         fsystem.setAngle(d);

         // Positions
         br.readLine();
         double x[][] = new double[numatoms][3];
         for (int i = 0; i < numatoms; i++){
            data = br.readLine().trim();
            tokens = data.split(" +");
            if (tokens.length != 3){
               return false;
            }
            x[i][0] = Double.parseDouble(tokens[0]);
            x[i][1] = Double.parseDouble(tokens[1]);
            x[i][2] = Double.parseDouble(tokens[2]);
         }
         List atoms = fsystem.getAtomList();
         for (Iterator i = atoms.iterator(); i.hasNext(); ){
            Atom a = (Atom) i.next();
            int j = a.getXYZIndex() - 1;
            a.moveto(x[j][0], x[j][1], x[j][2]);
         }

         // Velocities
         br.readLine();
         for (int i = 0; i < numatoms; i++){
            data = br.readLine().trim();
            tokens = data.split(" +");
            if (tokens.length != 3){
               return false;
            }
            x[i][0] = Double.parseDouble(tokens[0]);
            x[i][1] = Double.parseDouble(tokens[1]);
            x[i][2] = Double.parseDouble(tokens[2]);
         }
         double max = 0.0d;
         for (Iterator i = atoms.iterator(); i.hasNext(); ){
            Atom a = (Atom) i.next();
            int j = a.getXYZIndex() - 1;
            a.setVeclocity(x[j][0], x[j][1], x[j][2]);
            double r = AtomUtils.r(x[j]);
            if (r > max){
               max = r;
            }
         }
         Atom.setMaxVelocity(max);

         // Accelerations
         br.readLine();
         for (int i = 0; i < numatoms; i++){
            data = br.readLine().trim();
            tokens = data.split(" +");
            if (tokens.length != 3){
               return false;
            }
            x[i][0] = Double.parseDouble(tokens[0]);
            x[i][1] = Double.parseDouble(tokens[1]);
            x[i][2] = Double.parseDouble(tokens[2]);
         }
         max = 0.0d;
         for (Iterator i = atoms.iterator(); i.hasNext(); ){
            Atom a = (Atom) i.next();
            int j = a.getXYZIndex() - 1;
            a.setAcceleration(x[j][0], x[j][1], x[j][2]);
            double r = AtomUtils.r(x[j]);
            if (r > max){
               max = r;
            }
         }
         Atom.setMaxAcceleration(max);
         br.close();
         fr.close();
      } catch (Exception e){
         return false;
      }
      return true;
   }

}
