/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.parsers;

import java.io.*;
import java.util.*;

import ffe.lang.*;

/**
 * The InducedFilter class parses TINKER Induced Dipole (*.IND) files.
 */
public class InducedFilter{
   FSystem fsystem;
   File file;

   public InducedFilter(FSystem s, File f){
      fsystem = s;
      file = f;
   }

   public boolean read(){

      if (!file.exists() || !file.canRead()){
         return false;
      }
      try{
         FileReader fr = new FileReader(file);
         BufferedReader br = new BufferedReader(fr);

         String data = br.readLine();
         String tokens[] = data.trim().split(" +");
         if (tokens.length == 0){
            return false;
         }
         int numatoms = Integer.parseInt(tokens[0]);
         if (numatoms != fsystem.getAtomList().size()){
            return false;
         }

         // Read the Induced Dipoles
         double x[][] = new double[numatoms][3];
         for (int i = 0; i < numatoms; i++){
            data = br.readLine().trim();
            tokens = data.split(" +");
            if (tokens.length != 5){
               return false;
            }
            x[i][0] = Double.parseDouble(tokens[2]);
            x[i][1] = Double.parseDouble(tokens[3]);
            x[i][2] = Double.parseDouble(tokens[4]);
         }
         List atoms = fsystem.getAtomList();
         double max = 0.0d;
         for (Iterator i = atoms.iterator(); i.hasNext(); ){
            Atom a = (Atom) i.next();
            int j = a.getXYZIndex() - 1;
            a.setInducedDipole(x[j][0], x[j][1], x[j][2]);
            double r = AtomUtils.r(x[j]);
            if (r > max){
               max = r;
            }
         }
         Atom.setMaxInduced(max);
         br.close();
         fr.close();
      } catch (Exception e){
         return false;
      }

      return true;
   }
}
