/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides an GUI to TINKER</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.parsers;

import java.io.*;

import ffe.lang.*;

/**
 * The KeyFilter class parses TINKER Keyword (*.KEY) files.
 */
public class KeyFilter{

   public KeyFilter(){
   }

   public static boolean open(FSystem sys){
      if (sys == null){
         return false;
      }
      File keyFile = sys.getKeyFile();
      if (keyFile == null || !keyFile.exists() || !keyFile.canRead()){
         return false;
      }
      try{
         FileReader fr = new FileReader(keyFile);
         BufferedReader br = new BufferedReader(fr);
         Keyword comments = new Keyword("COMMENTS");
         sys.addKeyword(comments);

         while (br.ready()){
            String s = br.readLine();
            if (s == null){
               continue;
            }
            s = s.trim();
            if (s.equals("")){
               continue; //Skip blank lines
            }

            // Store comments together
            if (s.startsWith("#") || s.toUpperCase().startsWith("ECHO")){
               comments.append(s);
            } else{
               int firstspace = s.indexOf(" ");
               String keyword, data;
               if (firstspace == -1){ // no parameters
                  keyword = s.trim().toUpperCase();
                  data = null;
               } else{
                  keyword = s.substring(0, firstspace).toUpperCase();
                  data = s.substring(firstspace).trim();
               }

               Keyword kd = sys.getKeyword(keyword);
               if (kd == null){
                  kd = new Keyword(keyword);
                  sys.addKeyword(kd);
               }
               if (data != null){
                  kd.append(data);
               }

               if (keyword.equalsIgnoreCase("MULTIPOLE")){
                  int mnum[] = {3, 1, 2, 3};
                  for (int i = 0; i < 4; i++){
                     if (!br.ready()){
                        System.out.println("Check for invalid MULTIPOLE keyword at EOF");
                        return false;
                     }
                     s = br.readLine();
                     if (s == null){
                        System.out.println("Multipole format error ");
                        return false;
                     }
                     s = s.trim();
                     if (s.split(" +").length != mnum[i]){
                        System.out.println("Multipole format error ");
                        return false;
                     }
                  }
                  kd.append(s);
               }
            }
         }
         return true;
      } catch (IOException e){
         System.out.println("Error reading Key File: " + e);
         return false;
      }
   }

}
