/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.parsers;

import java.io.*;
import java.util.*;
import javax.media.j3d.SharedGroup;

import ffe.lang.*;
import ffe.panels.MainPanel;

//import com.sun.j3d.loaders.vrml97.VrmlLoader;

/**
 * The NEDFilter class parses "Network of Electrostatic Domain" (*.NED) files. This
 * is not currently being used.
 */
public final class NEDFilter
   extends SystemFilter{

   private static Hashtable mmatoms;

   public NEDFilter(FSystem f, MainPanel tink){
      super(f, null, tink);
   }

   public boolean writefile(){
      return false;
   }

   public boolean readfile(){
      String rawdata, altloc, atom_type, residue_type;
      String Polymer_type, FSystem_name = new String("PDB File");
      MMAtom mmatom;
      double[] d = new double[3];
      int residue_num;
      boolean vrmlopen = false;
      SharedGroup sg = null;
      Atom a = null;
      ArrayList links = new ArrayList();
      ArrayList structs = new ArrayList();

      File nedfile = fsystem.getFile();
      FSystem M = new FSystem(nedfile.getName(), true);
      M.setFile(nedfile);
      M.setFileType(SystemFilter.NED);

      try{
         // Try to read in a vrml file
         String name = new String(nedfile.getName());
         String wml = new String(name.substring(0, name.lastIndexOf(".")));
         wml = new String(wml + ".wrl");
         File wmlfile = new File(nedfile.getParent() + File.separatorChar + wml);
         System.out.println("VRML file: " + wmlfile.getAbsolutePath());

         /* if (wmlfile.exists()){
          VrmlLoader vrml = new VrmlLoader();
          BranchGroup bg = vrml.load(wmlfile.getAbsolutePath()).getSceneGroup();
          sg = new SharedGroup();
          sg.addChild(bg);
          M.setVrml(sg);
          vrmlopen = true;
          System.out.println("VRML file loaded");
             } */

         setFileRead(false);
         // Open a data stream to the NED file
         FileInputStream fin = new FileInputStream(nedfile);
         BufferedReader din = new BufferedReader(new InputStreamReader(fin));
         rawdata = din.readLine();
         int num = 1; //First atom is #1, to match xyz file format
         //while the END parameter is not read in, create atoms and residues
         while ((rawdata != null) && (!rawdata.startsWith("END"))){
            if (rawdata.startsWith("ATOM")){
               altloc = new String(rawdata.substring(16, 17));
               if (altloc.equals(" ") || altloc.equals("A")){
                  System.out.println("" + rawdata);
                  atom_type = new String(rawdata.substring(13, 16).trim());
                  residue_type = new String(rawdata.substring(17, 20).trim());
                  residue_num = Integer.decode(rawdata.substring(22, 26).trim()).intValue();
                  Polymer_type = new String(rawdata.substring(21, 22));
                  d[0] = Double.valueOf(rawdata.substring(30, 38).trim()).doubleValue();
                  d[1] = Double.valueOf(rawdata.substring(38, 46).trim()).doubleValue();
                  d[2] = Double.valueOf(rawdata.substring(46, 54).trim()).doubleValue();
                  mmatom = (MMAtom) mmatoms.get(atom_type);
                  if (mmatom == null){
                     System.out.println("MMAtom not found!");
                  }
                  //a = new Atom(atom_type, mmatom, d, 1,
                  //			 "ALA", Polymer_type, num++);
                  //a.print();
                  M.addFNode(a);
               }
            }
            rawdata = din.readLine();
         }
         fin.close();

         setFileRead(true);
         setFSystem(M);
      } catch (IOException e){
         System.out.println(e);
         setFileRead(false);
         return false;
      }
      return FileRead();
   }

   static{
      mmatoms = new Hashtable();
      mmatoms.put("CA", new MMAtom("End", "End", "Open End", 6, 1.0f, 1, 6.0f, 6.0f, 0.0f));
      mmatoms.put("N", new MMAtom("Domain", "Domain", "Domain", 7, 1.0f, 1, 6.0f, 6.0f, 0.0f));
      mmatoms.put("O", new MMAtom("End", "End", "Constrained End", 8, 1.0f, 1, 6.0f, 6.0f, 0.0f));
   }

}
