/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.parsers;

import java.util.*;

import ffe.lang.*;
import ffe.net.TinkerSystem;
import ffe.panels.MainPanel;

/**
 * The SocketFilter class parses system data sent by TINKER to Force Field Explorer.
 */
public final class SocketFilter
   extends SystemFilter{

   TinkerSystem system;
   Hashtable atomtypes = new Hashtable();

   public SocketFilter(MainPanel m, TinkerSystem sys){
      super(null, null, m);
      system = sys;
      setType(SystemFilter.SIM);
      setFileRead(false);
   }

   public boolean writefile(){
      return false;
   }

   public boolean readfile(){
      FSystem M = new FSystem(system.file.trim(), true);
      M.setMMFilter(null);
      M.setFile(null);
      M.setKeyFile(null);
      M.setFileType(SystemFilter.SIM);
      M.setForceField(system.forcefield.trim());
      setFSystem(M);

      Integer b1, b2;
      int i, j, b[] = new int[4];
      double d[] = new double[3];
      MMAtom mmatom;
      String s;
      Vector bonds1 = new Vector();
      Vector bonds2 = new Vector();
      Atom a;

      // Create Molecular Mechanics Data Objects from the TinkerSystem information
      for (i = 0; i < system.numatoms; i++){
         s = Integer.toString(system.types[i]);
         mmatom = (MMAtom) atomtypes.get(s);
         if (mmatom == null){
            mmatom = new MMAtom(s, system.name[i], system.story[i], system.atomic[i],
                     (float) system.mass[i], 0, 1.0f, 1.0f, (float) system.charge[i]);
            atomtypes.put(s, mmatom);
         }
      }

      atomList = new ArrayList();
      for (i = 0; i < system.numatoms; i++){
         d[0] = (double) system.coordinates[0][i];
         d[1] = (double) system.coordinates[1][i];
         d[2] = (double) system.coordinates[2][i];
         s = new String("" + system.types[i]);
         mmatom = (MMAtom) atomtypes.get(s);
         a = new Atom(i + 1, mmatom.type, mmatom, d);
         atomList.add(a);
         b1 = new Integer(i + 1);
         b[0] = system.connectivity[0][i];
         b[1] = system.connectivity[1][i];
         b[2] = system.connectivity[2][i];
         b[3] = system.connectivity[3][i];
         j = 0;
         while (j < 4 && b[j] != 0){
            b2 = new Integer(b[j]);
            bonds1.add(b1);
            bonds2.add(b2);
            j++;
         }

      }

      int num_bonds = bonds1.size();
      int a1, a2;
      Atom atom1, atom2;

      bondList = new ArrayList();
      for (i = 0; i < num_bonds; i++){
         a1 = ((Integer) bonds1.get(i)).intValue();
         a2 = ((Integer) bonds2.get(i)).intValue();
         if (a1 < a2){
            atom1 = (Atom) atomList.get(a1 - 1);
            atom2 = (Atom) atomList.get(a2 - 1);
            bondList.add(new Bond(atom1, atom2, 1));
         }
      }

      setFileRead(true);
      return true;
   }
}
