/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.parsers;

import java.io.File;
import java.util.ArrayList;

import java.awt.Cursor;
import javax.swing.JLabel;

import ffe.lang.*;
import ffe.panels.MainPanel;

/**
 * The SystemFilter class is the base class for most file parsers.
 */
public abstract class SystemFilter
   implements Runnable{

   protected FSystem fsystem = null;

   /**
    * The atoms and bonds ArrayLists are filled by the filters that extend this base class.
    */
   protected ArrayList atomList;
   protected ArrayList bondList;

   protected MMFilter mmfilter;
   protected boolean file_read;
   protected byte filetype = 0;
   protected MainPanel fpanel;
   protected JLabel statusBar;

   public SystemFilter(){
      file_read = false;
   }

   public SystemFilter(FSystem f, MainPanel m){
      fsystem = f;
      fpanel = m;
      statusBar = fpanel.getStatusBar();
   }

   public SystemFilter(FSystem f, MMFilter mm, MainPanel m){
      this(f, m);
      mmfilter = mm;
   }

   /**
    * This method is different for each subclass
    * and must be overidden
    */
   public abstract boolean readfile();

   public abstract boolean writefile();

   /**
    * The open method trys to open a file picked from a JFileChooser.
    * A successful read is followed by attachment of the FSystem to the root node
    * and the update of the various displays
    */
   private void open(){
      /*Runtime runtime = Runtime.getRuntime();
               long occupiedMemory = 0;
               occupiedMemory = runtime.totalMemory() - runtime.freeMemory();
               System.out.println("Total Memory    (kb):    " +
                         runtime.totalMemory() / 1024);
               System.out.println("Free Memory     (kb):     " +
                         runtime.freeMemory() / 1024);
       System.out.println("Occupied Memory (kb): " + occupiedMemory / 1024);*/

      // Continue if the file was read in successfully
      if (readfile()){
         if (fsystem.getFileType() != SystemFilter.PDB){
            statusBar.setText("  Determing Structure");
            XYZUtils.groupXYZ(getFSystem(), atomList, fpanel);
         }
         statusBar.setText("  Rendering Wireframe");
         Root root = fpanel.getRoot();
         root.addFSystem(fsystem);
         fsystem.finalize(true);
         fpanel.getTree().setActive(fsystem);
         fpanel.update(true);

         setFileRead(true);
      } else{
         // fpanel.update(false);
      }
      fpanel.revalidate();

      /*
       occupiedMemory = (runtime.totalMemory() - runtime.freeMemory()) -
          occupiedMemory;
               if (m != null) {
          System.out.println("Memory for " + m.getName() + " (kb): " +
                             occupiedMemory / 1024);

               }*/

      fpanel.setCursor(Cursor.getDefaultCursor());
   }

   public void run(){
      open();
   }

   /**
    * Return the FSystem that has been read in
    */
   public FSystem getFSystem(){
      return fsystem;
   }

   public void setFSystem(FSystem f){
      fsystem = f;
   }

   /**
    * Returns true if the read was successful
    */
   public boolean FileRead(){
      return file_read;
   }

   public void setFileRead(boolean b){
      file_read = b;
   }

   public void setType(byte t){
      filetype = t;
   }

   public byte getType(){
      return filetype;
   }

   public static final byte XYZ = 1;
   public static final byte INT = 2;
   public static final byte ARC = 4;
   public static final byte PDB = 8;
   public static final byte SIM = 16;
   public static final byte NED = 32;

   public static String toFileString(byte filetype){
      if (filetype == 1){
         return new String("XYZ").intern();
      } else if (filetype == 2){
         return new String("INT").intern();
      } else if (filetype == 4){
         return new String("ARC").intern();
      } else if (filetype == 8){
         return new String("PDB").intern();
      } else if (filetype == 16){
         return new String("SIM").intern();
      } else{
         return null;
      }
   }

   public static File version(File file){
      if (file == null){
         return null;
      }

      if (!file.exists()){
         return file;
      }

      String fileName = file.getAbsolutePath();
      int dot = file.getAbsolutePath().lastIndexOf(".");
      int under = file.getAbsolutePath().lastIndexOf("_");

      File newFile = file;
      if (under > dot){
         String name = fileName.substring(0, under);
         newFile = new File(name);
      }
      File oldFile = newFile;

      int i = 1;
      while (newFile.exists()){
         i = i + 1;
         newFile = oldFile;
         int thousand = i / 1000;
         int hundred = (i - 1000 * thousand) / 100;
         int tens = (i - 1000 * thousand - 100 * hundred) / 10;
         int ones = i - 1000 * thousand - 100 * hundred - 10 * tens;
         StringBuffer newFileString = new StringBuffer(oldFile.getAbsolutePath());
         if (thousand != 0){
            newFileString.append('_').append(thousand).append(hundred).append(tens).append(ones);
         } else if (hundred != 0){
            newFileString.append('_').append(hundred).append(tens).append(ones);
         } else if (tens != 0){
            newFileString.append('_').append(tens).append(ones);
         } else{
            newFileString.append('_').append(ones);
         }
         newFile = new File(newFileString.toString());
      }
      return newFile;
   }
}
