/**
 * <p>Title: Force Field Explorer</p>
 * <p>Description: Force Field Explorer provides a user interface to TINKER</p>
 * <p>Copyright: Copyright (c) 2004 Jay Ponder</p>
 * <p>Institution: Washington University</p>
 * @author Michael J. Schnieders
 * @version 4.2
 */

package ffe.platforms.macosx;

import com.apple.eawt.*;

import ffe.panels.MainPanel;

/**
 * The OSXAdapter class was developed by following an example supplied on the OS X site. It handles
 * events generated by the following standard OS X toolbar items:
 * About, Preferences, Quit and File Associations
 */
public class OSXAdapter
   extends ApplicationAdapter{

   private static OSXAdapter osxAdapter;
   private static Application osxApplication;
   private MainPanel mainPanel;

   private OSXAdapter(MainPanel m){
      mainPanel = m;
   }

   public static void registerMacOSXApplication(MainPanel m){
      if (osxApplication == null){
         osxApplication = new Application();
      }
      if (osxAdapter == null){
         osxAdapter = new OSXAdapter(m);
      }
      osxApplication.addApplicationListener(osxAdapter);
      osxApplication.setEnabledPreferencesMenu(true);
      osxApplication.setEnabledAboutMenu(true);
   }

   public void handleAbout(ApplicationEvent ae){
      if (mainPanel != null){
         ae.setHandled(true);
         mainPanel.about();
      } else{
         ae.setHandled(false);
      }
   }

   public void handlePreferences(ApplicationEvent ae){
      if (mainPanel != null){
         mainPanel.preferences();
         ae.setHandled(true);
      } else{
         ae.setHandled(false);
      }
   }

   public void handleQuit(ApplicationEvent ae){
      if (mainPanel != null){
         ae.setHandled(false);
         mainPanel.exit();
      } else{
         System.exit( -1);
      }
   }

   public void handleOpenFile(ApplicationEvent ae){
      if (mainPanel != null){
         mainPanel.open(ae.getFilename());
         ae.setHandled(true);
      } else{
         ae.setHandled(false);
      }
   }

}
